package cz.muni.fi.pa165.rest.resources;
 
import java.net.URI;
import java.util.SortedMap;
import java.util.concurrent.ConcurrentSkipListMap;
import java.util.concurrent.atomic.AtomicInteger;
import javax.annotation.security.RolesAllowed;
import javax.inject.Singleton;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriInfo;
import javax.ws.rs.PathParam;
import javax.ws.rs.Path;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
 
/**
 * Note: we are instantiating all the customers and adding them to 
 * an in-memory representation
 *
 */
@Path("/customers")
@Singleton
public class CustomersResource {
 
    private static SortedMap<Integer, CustomerResource> customerDB = new ConcurrentSkipListMap<Integer, CustomerResource>();
  
    private static AtomicInteger idCounter = new AtomicInteger(0);
    
    @Context
    private UriInfo context;
 
    public CustomersResource() {
           
        customerDB.put(idCounter.incrementAndGet(), new CustomerResource(Integer.toString(idCounter.get()), 
                "Isaac", "Newton", "Scientist", "Law of Universal Gravitation"));
        customerDB.put(idCounter.incrementAndGet(), new CustomerResource(Integer.toString(idCounter.get()), 
                "Johannes", "Gutenberg", "Inventor", "printing press"));
        customerDB.put(idCounter.incrementAndGet(), new CustomerResource(Integer.toString(idCounter.get()), 
                "Albert", "Einstein", "Scientist", "Theory of Relativity"));
        customerDB.put(idCounter.incrementAndGet(), new CustomerResource(Integer.toString(idCounter.get()), 
                "Enrico", "Fermi", "Physicist", "Development of Quantum Theory"));
        customerDB.put(idCounter.incrementAndGet(), new CustomerResource(Integer.toString(idCounter.get()), 
                "James", "Watt", "Inventor", "Steam Engine"));
        customerDB.put(idCounter.incrementAndGet(), new CustomerResource(Integer.toString(idCounter.get()), 
                "Guglielmo", "Marconi", "Inventor", "Radio"));
        customerDB.put(idCounter.incrementAndGet(), new CustomerResource(Integer.toString(idCounter.get()), 
                "Werner", "Eisenberg", "Physicist", "Quantum Mechanics"));
        
    }
 
    /**
     * Return textual representation of the Customers resource
     * @return Customers as string
     */
    @GET
    @Produces(MediaType.TEXT_PLAIN)
    public String getPlain() {
        StringBuilder returnString = new StringBuilder();
 
        for (CustomerResource customerResource : customerDB.values()) {
            returnString.append(customerResource);
            returnString.append(" ");
        }
 
        return returnString.toString();
    }
 
    /**
     * Get a specific Customer resource
     * @param id of the Customer to retrieve
     * @return 
     */
    @GET
    @Path("{id}")
    @Produces(MediaType.TEXT_PLAIN)
    public String getCustomerResource(@PathParam("id") Integer id) {
        return customerDB.get(id).toString();
    }
 
    /**
     * counter for the number of Customers
     * @return integer count
     */
    @GET
    @Path("count")
    @Produces(MediaType.TEXT_PLAIN)
    public String getCount() {
        return String.valueOf(customerDB.size());
    }
    
    /**
     * Get representation of a CustomerResource as JSON
     * @param id - identifier for the CustomerResource
     * @return JSON representation of CustomerResource
     */
    @GET
    @Path("{id}")
    @Produces(MediaType.APPLICATION_JSON)
    public CustomerResource getJson(@PathParam("id") Integer id) {
        return customerDB.get(id);
    }
    
    /**
     * POST one new CustomerResource
     * @param customerResource - the new CustomerResource to add
     * @return Response created
     */
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    public Response postJson(CustomerResource customerResource) {
        customerDB.put(Integer.parseInt(customerResource.getId()), customerResource);
        System.out.println("Created customer " + customerResource.getId());
        return Response.created(URI.create(context.getAbsolutePath() + "/"+ customerResource.getId())).build();
    }
    
    /**
     * PUT for CustomerResource(s)
     * @param id - identifier for the resource
     * @param customerResource - the CustomerResource to put
     * @return Response created/no content
     */
    @PUT
    @Path("{id}")
    @Consumes(MediaType.APPLICATION_JSON)
    public Response putJson(@PathParam("id") Integer id, CustomerResource customerResource) {

        System.out.println("----  putting item ");

        URI uri = context.getAbsolutePath();
        System.out.println(context.getAbsolutePath());

        Response response;
        if (!customerDB.containsKey(id)) {
            response = Response.created(uri).build();
        } else {
            response = Response.noContent().build();
        }

        customerDB.put(id, new CustomerResource(customerResource));

        return response;
    }
    
    
    /**
     * DELETE a CustomerResource, returns 200 OK or 404 not found
     * @param id - identifier for the CustomerResource to delete 
     */
    @DELETE
    @Path("{id}")
    public void delete(@PathParam("id") Integer id) {
        System.out.println("---- Deleting item nr. " + id);

        if (!customerDB.containsKey(id)) {
            throw new WebApplicationException(Response.Status.NOT_FOUND);
        }
        customerDB.remove(id);
    }
}